"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tcp_client_1 = require("./libs.android/tcp-client");
var util_1 = require("./libs.android/util");
var ElectrumxClient = (function (_super) {
    __extends(ElectrumxClient, _super);
    function ElectrumxClient(host, port, options) {
        return _super.call(this, host, port, options) || this;
    }
    ElectrumxClient.prototype._request = function (method, params) {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                if (!this.status)
                    return [2, Promise.reject(new Error('ESOCKET'))];
                return [2, new Promise(function (resolve, reject) { return __awaiter(_this, void 0, void 0, function () {
                        var id, content;
                        return __generator(this, function (_a) {
                            switch (_a.label) {
                                case 0:
                                    id = ++this.atomicInteger;
                                    content = util_1.makeRequest(method, params, id);
                                    this.callbackQueue[id] = util_1.createPromiseResult(resolve, reject);
                                    return [4, this.send(content + '\n')];
                                case 1:
                                    _a.sent();
                                    return [2];
                            }
                        });
                    }); })];
            });
        });
    };
    ElectrumxClient.prototype.close = function () {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            var list;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, _super.prototype.close.call(this)];
                    case 1:
                        _a.sent();
                        list = [
                            'blockchain.headers.subscribe',
                            'blockchain.address.subscribe'
                        ];
                        list.forEach(function (eventName) { return _this.subscribe.removeAllListeners(eventName); });
                        return [2, Promise.resolve(true)];
                }
            });
        });
    };
    ElectrumxClient.prototype.server_version = function (client_name, protocol_version) {
        return this._request('server.version', [client_name, protocol_version]);
    };
    ElectrumxClient.prototype.server_banner = function () {
        return this._request('server.banner', []);
    };
    ElectrumxClient.prototype.server_ping = function () {
        return this._request('server.ping', []);
    };
    ElectrumxClient.prototype.server_addPeer = function (features) {
        return this._request('server.add_peer', [features]);
    };
    ElectrumxClient.prototype.serverDonation_address = function () {
        return this._request('server.donation_address', []);
    };
    ElectrumxClient.prototype.serverPeers_subscribe = function () {
        return this._request('server.peers.subscribe', []);
    };
    ElectrumxClient.prototype.blockchainScripthash_getBalance = function (scripthash) {
        return this._request('blockchain.scripthash.get_balance', [scripthash]);
    };
    ElectrumxClient.prototype.blockchainScripthash_getHistory = function (scripthash) {
        return this._request('blockchain.scripthash.get_history', [scripthash]);
    };
    ElectrumxClient.prototype.blockchainScripthash_history = function (scripthash, start_height) {
        return this._request('blockchain.scripthash.history', [scripthash, start_height]);
    };
    ElectrumxClient.prototype.blockchainScripthash_utxos = function (scripthash, start_height) {
        return this._request('blockchain.scripthash.utxos', [scripthash, start_height]);
    };
    ElectrumxClient.prototype.blockchainScripthash_getMempool = function (scripthash) {
        return this._request('blockchain.scripthash.get_mempool', [scripthash]);
    };
    ElectrumxClient.prototype.blockchainScripthash_listunspent = function (scripthash) {
        return this._request('blockchain.scripthash.listunspent', [scripthash]);
    };
    ElectrumxClient.prototype.blockchainScripthash_subscribe = function (scripthash) {
        return this._request('blockchain.scripthash.subscribe', [scripthash]);
    };
    ElectrumxClient.prototype.blockchainBlock_header = function (height, cp_height) {
        return this._request('blockchain.block.header', [height, cp_height]);
    };
    ElectrumxClient.prototype.blockchainBlock_headers = function (start_height, count) {
        return this._request('blockchain.block.headers', [start_height, count]);
    };
    ElectrumxClient.prototype.blockchainEstimatefee = function (numBlocks) {
        return this._request('blockchain.estimatefee', [numBlocks]);
    };
    ElectrumxClient.prototype.blockchainHeaders_subscribe = function () {
        return this._request('blockchain.headers.subscribe', [false]);
    };
    ElectrumxClient.prototype.blockchain_relayfee = function () {
        return this._request('blockchain.relayfee', []);
    };
    ElectrumxClient.prototype.blockchainTransaction_broadcast = function (rawtx) {
        return this._request('blockchain.transaction.broadcast', [rawtx]);
    };
    ElectrumxClient.prototype.blockchainTransaction_get = function (txHash, verbose, merkle) {
        return this._request('blockchain.transaction.get', [
            txHash, (verbose ? verbose : false), (merkle ? merkle : false)
        ]);
    };
    ElectrumxClient.prototype.blockchainTransaction_getMerkle = function (txHash, height) {
        return this._request('blockchain.transaction.get_merkle', [txHash, height]);
    };
    ElectrumxClient.prototype.mempool_getFeeHistogram = function () {
        return this._request('mempool.get_fee_histogram', []);
    };
    ElectrumxClient.prototype.blockchainBlock_getHeader = function (height) {
        return this._request('blockchain.block.get_header', [height]);
    };
    ElectrumxClient.prototype.blockchainBlock_getChunk = function (index) {
        return this._request('blockchain.block.get_chunk', [index]);
    };
    ElectrumxClient.prototype.blockchainAddress_getBalance = function (address) {
        return this._request('blockchain.address.get_balance', [address]);
    };
    ElectrumxClient.prototype.blockchainAddress_getHistory = function (address) {
        return this._request('blockchain.address.get_history', [address]);
    };
    ElectrumxClient.prototype.blockchainAddress_getMempool = function (address) {
        return this._request('blockchain.address.get_mempool', [address]);
    };
    ElectrumxClient.prototype.blockchainAddress_listunspent = function (address) {
        return this._request('blockchain.address.listunspent', [address]);
    };
    ElectrumxClient.prototype.blockchainAddress_subscribe = function (address) {
        return this._request('blockchain.address.subscribe', [address]);
    };
    ElectrumxClient.prototype.blockchainUtxo_getAddress = function (tx_hash, index) {
        return this._request('blockchain.utxo.get_address', [tx_hash, index]);
    };
    ElectrumxClient.prototype.blockchainNumblocks_subscribe = function () {
        return this._request('blockchain.numblocks.subscribe', []);
    };
    return ElectrumxClient;
}(tcp_client_1.TcpClient));
exports.ElectrumxClient = ElectrumxClient;
//# sourceMappingURL=electrumx-client.android.js.map