"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var UnexpectedResponseError = (function (_super) {
    __extends(UnexpectedResponseError, _super);
    function UnexpectedResponseError(rawResponse) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        var _this = _super.apply(this, args) || this;
        _this.name = 'UnexpectedResponseError';
        _this.message = args[0];
        _this.rawResponse = rawResponse;
        return _this;
    }
    return UnexpectedResponseError;
}(Error));
var TCPClientError = (function (_super) {
    __extends(TCPClientError, _super);
    function TCPClientError(rawResponse) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        var _this = _super.apply(this, args) || this;
        _this.name = 'TCPClientError';
        _this.message = args[0];
        return _this;
    }
    return TCPClientError;
}(Error));
var EventEmitter = require("events");
var TcpClient = (function () {
    function TcpClient(host, port, options) {
        if (typeof options === 'undefined')
            options = {};
        this.host = host;
        this.port = port;
        this.status = 0;
        this.atomicInteger = 0;
        this.callbackQueue = {};
        this.subscribe = new EventEmitter();
        var self = this;
        var listener = new cz.honzamrazek.simplenetworking.TcpClientListener({
            onData: function (data) {
                var dataArr = data.split('\n');
                dataArr.pop();
                dataArr.forEach(function (_raw) {
                    try {
                        var data_1 = JSON.parse(_raw);
                        if (data_1.id !== undefined)
                            self.handleResponse(data_1);
                        else
                            self.subscribe.emit(data_1.method, data_1.params);
                    }
                    catch (err) {
                        self.subscribe.emit('error', new UnexpectedResponseError(_raw, 'Invalid JSON'));
                    }
                    self.subscribe.emit('data', _raw);
                });
            },
            onError: function (id, message) {
                self.subscribe.emit('error', new TCPClientError("TCPClient encountered an issue with (#" + id + "):" + message));
            },
            onFinished: function (id) {
                self.subscribe.emit('finished', id);
            }
        });
        this.client = new cz.honzamrazek.simplenetworking.TcpClient(listener);
    }
    TcpClient.prototype.connect = function () {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            var callback;
            return __generator(this, function (_a) {
                return [2, new Promise(function (resolve, reject) { return __awaiter(_this, void 0, void 0, function () {
                        var id;
                        return __generator(this, function (_a) {
                            switch (_a.label) {
                                case 0:
                                    if (this.status)
                                        return [2, resolve(true)];
                                    this.status = 1;
                                    return [4, this.client.start(this.host, this.port)];
                                case 1:
                                    id = _a.sent();
                                    setTimeout(function () {
                                        resolve(id);
                                    }, 2000);
                                    return [2];
                            }
                        });
                    }); })];
            });
        });
    };
    TcpClient.prototype.close = function () {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.status)
                            return [2, true];
                        this.status = 0;
                        Object.keys(this.callbackQueue).forEach(function (key) {
                            _this.callbackQueue[key](new Error('close connect'));
                            delete _this.callbackQueue[key];
                        });
                        return [4, this.client.stop()];
                    case 1:
                        _a.sent();
                        return [2, Promise.resolve()];
                }
            });
        });
    };
    TcpClient.prototype.send = function (content) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2, this.client.send(content)];
            });
        });
    };
    TcpClient.prototype.handleResponse = function (data) {
        var callback = this.callbackQueue[data.id];
        if (!callback)
            return;
        delete this.callbackQueue[data.id];
        if (data.error)
            callback(data.error);
        else
            callback(null, data.result);
    };
    return TcpClient;
}());
exports.TcpClient = TcpClient;
//# sourceMappingURL=tcp-client.js.map